@@ -1,6 +1,6 @@
 # $Author: ddumont $
-# $Date: 2008-03-11 13:41:37 +0100 (Tue, 11 Mar 2008) $
-# $Revision: 537 $
+# $Date: 2008-03-22 19:10:39 +0100 (Sat, 22 Mar 2008) $
+# $Revision: 558 $
 
 #    Copyright (c) 2008 Dominique Dumont.
 #
@@ -30,7 +30,7 @@ use base qw/Tk::Frame Config::Model::Tk::AnyViewer/;
 use vars qw/$VERSION/ ;
 use subs qw/menu_struct/ ;
 
-$VERSION = sprintf "1.%04d", q$Revision: 537 $ =~ /(\d+)/;
+$VERSION = sprintf "1.%04d", q$Revision: 558 $ =~ /(\d+)/;
 
 Construct Tk::Widget 'ConfigModelListViewer';
 
@@ -65,7 +65,9 @@ sub Populate {
 				     -height => 10,
 				   ) ->pack(@fbe1) ;
 
-    foreach my $c ($list->get_all_indexes) {
+    my @insert = $list->cargo_type eq 'leaf' ? $list->fetch_all_values 
+               :                         $list->get_all_indexes ;
+    foreach my $c (@insert) {
 	$rt->insert('end', $c."\n" ) ;
     }
 
@@ -85,8 +87,7 @@ sub add_info {
 
     my $list = $cw->{list} ;
 
-    my @items = ('type : '. $list->get_type 
-                          . ( $list->ordered ? '(ordered)' : ''),
+    my @items = ('type : '. $list->get_type ,
 		 'index : '.$list->index_type ,
 		 'cargo : '.$list->cargo_type ,
 		);
@@ -95,7 +96,7 @@ sub add_info {
 	push @items, "cargo class: " . $list->config_class_name ;
     }
 
-    foreach my $what (qw/min max max_nb/) {
+    foreach my $what (qw//) {
 	my $v = $list->$what() ;
 	my $str = $what ;
 	$str =~ s/_/ /g;
